#'Compute Bias Corrected Climatologies
#'
#'This function computes only per-pair climatologies from the experimental 
#'and observational matrices output from \code{Load()}.
#'To compute plain climatologies from only experimental or observational 
#'data from \code{Load()}, the following code can be used:\cr 
#'\code{clim <- array(apply(obs_data, c(1, 4, 5, 6), mean),}\cr
#'\code{              dim = dim(obs_datta)[-c(2, 3)])}\cr
#'The function \code{Clim()} computes per-pair climatologies using one of the 
#'following methods:
#'\enumerate{
#'  \item{per-pair method (Garcia-Serrano and Doblas-Reyes, CD, 2012)}
#'  \item{Kharin method (Karin et al, GRL, 2012)}
#'  \item{Fuckar method (Fuckar et al, GRL, 2014)}
#'}
#'\code{Clim()} computes climatologies using the startdates covered by the 
#'whole experiments/observational data sets. The startdates not available for 
#'all the data (model and obs) are excluded when computing the climatologies.
#'
#'@param exp Model data: c(nmod/nexp, nmemb/nparam, nsdate, nltime) up to 
#'  c(nmod/nexp, nmemb/nparam, nsdate, nltime, nlevel, nlat, nlon).
#'@param obs Observational data: c(nobs, nmemb, nsdate, nltime) up to 
#'  c(nobs, nmemb, nsdate, nltime, nlevel, nlat, nlon).
#'@param memb TRUE/FALSE (1 climatology for each member). Default = TRUE.
#'@param kharin TRUE/FALSE (if Kharin method is applied or not). 
#'  Default = FALSE.
#'@param NDV TRUE/FALSE (if Fuckar method is applied or not). Default = FALSE.
#'
#'@return
#'\item{clim_exp}{Array with same dimensions as exp except the third 
#'  (starting dates) and, depending on the parameters, the second (members), 
#'  which disappear.}
#'\item{clim_obs}{Array with same dimensions as obs except the third 
#'  (starting dates) and, depending on the parameters, the second (members), 
#'  which disappear.}
#'@keywords datagen
#'@author History:\cr
#'  0.9 - 2011-03 (V. Guemas, \email{virginie.guemas@@ic3.cat}) - Original code\cr
#'  1.0 - 2013-09 (N. Manubens, \email{nicolau.manubens@@ic3.cat}) - Formatting to R CRAN
#'@examples
#'# Load sample data as in Load() example:
#'example(Load)
#'clim <- Clim(sampleData$mod, sampleData$obs)
#'  \donttest{
#'PlotClim(clim$clim_exp, clim$clim_obs, 
#'         toptitle = paste('sea surface temperature climatologies'), 
#'         ytitle = 'K', monini = 11, listexp = c('CMIP5 IC3'), 
#'         listobs = c('ERSST'), biglab = FALSE, fileout = 'tos_clim.eps')
#'  }
#'@importFrom abind adrop
#'@import multiApply
#'@export
Clim <- function(exp, obs, memb = TRUE, method = 'clim', dat_dim = NULL, comp_dim = NULL, limits = NULL,
                 sdate_dim = 'sdate', memb_dim = 'member', na.rm = TRUE, ncores = NULL) {


  ###############################
  # Calculate Clim

  #----------------------------------
  #  Remove data along comp_dim dim if there is at least one NA between limits
  if (!is.null(comp_dim)) {
    
    if (is.null(limits)) {
      limits <- list(NA, length(comp_dim))
      for (ii in 1:length(comp_dim)) { 
        limits[[ii]] <- c(1, dim(obs)[comp_dim[ii]])
      }
    }

    for (i in 1:length(comp_dim)) {  #[dat, sdate]

    pos <- which(names(dim(obs)) == comp_dim[i])
    outrows <- is.na(Mean1Dim(obs, pos, narm = FALSE, limits[[i]]))
    outrows <- InsertDim(outrows, pos, dim(obs)[comp_dim[i]])
    obs[which(outrows)] <- NA
    }
  }
  #-----------------------------------


  if (method == 'clim') {
    clim <- Apply(list(exp, obs), 
                  target_dims = c(sdate_dim, memb_dim, dat_dim),
                  fun = .Clim, 
                  method = method, na.rm = na.rm, memb = memb, ncores = ncores)
    # Add member dimension name back
    if (memb) {
      if(is.null(names(dim(clim$clim_exp))[1])) {
        names(dim(clim$clim_exp))[1] <- memb_dim
        names(dim(clim$clim_obs))[1] <- memb_dim
      }
    }

  } else if (method == 'kharin') {
    clim <- Apply(list(exp, obs),
                  target_dims = c(sdate_dim, memb_dim, dat_dim),
                  fun = .Clim,
                  method = method, na.rm = na.rm, memb = memb, ncores = ncores)
  }


  return(clim)
}


.Clim <- function(exp, obs, #sdate_dim = 'sdate', memb_dim = 'member', dat_dim2 = dat_dim, 
                   method = method, na.rm = TRUE, memb = TRUE) {

 ## 1. sdate mean. member average is done after this 

  # exp: [sdate, member_exp]
  # obs: [sdate, member_obs]

  if (method == 'clim') {
    clim_exp <- apply(exp, which(names(dim(exp)) != sdate_dim),
                      mean, na.rm = na.rm) #average out sdate_dim
    clim_obs <- apply(obs, which(names(dim(obs)) != sdate_dim),
                      mean, na.rm = na.rm) #[memb] or [memb, dat]

  } else if (method == 'kharin') {
  # obs clim
    clim_obs <- apply(obs, which(names(dim(obs)) != sdate_dim), 
                      mean, na.rm = na.rm) #[memb] or [memb, dat]

  # exp clim
  ##--- NEW trend ---##
source('/home/Earth/aho/s2dverification/R/Trend.R')
    tmp_obs <- Trend(data = obs, sdate_dim = sdate_dim, interval = 1, 
                     polydeg = 1, conf = FALSE, siglev = 0.95)$trend
    tmp_exp <- Trend(data = exp, sdate_dim = sdate_dim, interval = 1, 
                     polydeg = 1, conf = FALSE, siglev = 0.95)$trend
    # tmp_exp: [stats, member_exp, (dat_exp)]

    trend_obs <- array(dim = dim(exp))  #[sdate, member_exp, (dat_exp)]
    trend_exp <- array(dim = dim(exp))

    tmp_obs_mean <- apply(tmp_obs, 1, mean)  #average out member (and dataset)
    tmp_obs_mean <- InsertDim(tmp_obs_mean, 2, dim(exp)[memb_dim])
    if (length(dim(exp)) == 3) {
    tmp_obs_mean <- InsertDim(tmp_obs_mean, 3, dim(exp)[dat_dim])
    }

    for (jdate in 1:dim(exp)[sdate_dim]) {
      if (length(dim(exp)) == 2) {
        trend_exp[jdate, ] <- tmp_exp[1, ] + jdate * tmp_exp[2, ]
        trend_obs[jdate, ] <- tmp_obs_mean[1, ] + jdate * tmp_obs_mean[2, ]
      } else if (length(dim(exp)) == 3) {
        trend_exp[jdate, , ] <- tmp_exp[1, , ] + jdate * tmp_exp[2, , ]
        trend_obs[jdate, , ] <- tmp_obs_mean[1, , ] + jdate * tmp_obs_mean[2, , ]
      }
    }

    if (length(dim(exp)) == 3) {
      clim_obs_mean <- apply(clim_obs, 1, mean) 
      clim_obs_mean <- mean(clim_obs_mean)
      clim_obs_mean <- InsertDim(
                         InsertDim(
                           InsertDim(clim_obs_mean, 1, dim(exp)[1]), 
                           2, dim(exp)[2]), 
                         3, dim(exp)[3])  #[sdate, memb_exp, dat_exp, 1]
      clim_obs_mean <- Subset(clim_obs_mean, 4, 1, drop = 'selected')  #[sdate, memb_exp, dat_exp]

    } else if (length(dim(exp)) == 2) {
      clim_obs_mean <- mean(clim_obs)  #a number
      clim_obs_mean <- InsertDim(
                           InsertDim(clim_obs_mean, 1, dim(exp)[1]),
                           2, dim(exp)[2])   #[sdate, memb_exp, 1]
      clim_obs_mean <- Subset(clim_obs_mean, 3, 1, drop = 'selected')  #[sdate, memb_exp]
    }
    
    clim_exp <- trend_exp - trend_obs + clim_obs_mean
    

  } else if (method == 'NDV') {
        #nose aun
  }

 ## 2. member mean
  if (!memb) {
    clim_exp <- mean(clim_exp, na.rm = TRUE) #remove NA member
    clim_obs <- mean(clim_obs, na.rm = TRUE) #return a number
  }

    return(list(clim_exp = clim_exp, clim_obs = clim_obs))
}
